﻿using System;
using System.Configuration;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Security.Cryptography.X509Certificates;

namespace FMK.EFH.API.Sample
{
    class ClientFactory
    {

        public static HttpClient CreateForAgent()
        {
            var handler = new WebRequestHandler();
            handler.ClientCertificateOptions = ClientCertificateOption.Manual;
            handler.ClientCertificates.Add(GetClientCertificate());
            
            var client = new HttpClient(handler);

            client.BaseAddress = new Uri(ConfigurationManager.AppSettings["FMK.EFH.API.BaseAddress"]);

            var identity = ConfigurationManager.AppSettings["FMK.EFH.API.Identifier.Agent"];
            if (!string.IsNullOrEmpty(identity))
            {
                client.DefaultRequestHeaders.Add("X-Identifier", identity);
            }
            
            client.DefaultRequestHeaders.Accept.Clear();
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("text/xml"));

            return client;

        }

        public static HttpClient CreateForPresenteeParty()
        {
            var handler = new WebRequestHandler();
            handler.ClientCertificateOptions = ClientCertificateOption.Manual;
            handler.ClientCertificates.Add(GetClientCertificate());

            var client = new HttpClient(handler);

            client.BaseAddress = new Uri(ConfigurationManager.AppSettings["FMK.EFH.API.BaseAddress"]);

            var identity = ConfigurationManager.AppSettings["FMK.EFH.API.Identifier.PresenteeParty"];
            if (!string.IsNullOrEmpty(identity))
            {
                client.DefaultRequestHeaders.Add("X-Identifier", identity);
            }

            client.DefaultRequestHeaders.Accept.Clear();
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("text/xml"));

            return client;

        }

        public static X509Certificate2 GetClientCertificate()
        {
            var thumbprint = ConfigurationManager.AppSettings["FMK.EFH.API.ClientCertificateThumbprint"];

            var cert = TryGetClientCertificateFromStore(StoreLocation.CurrentUser, thumbprint);
            if (cert == null)
            {
                cert = TryGetClientCertificateFromStore(StoreLocation.LocalMachine, thumbprint);
            }

            if (cert != null)
                return cert;

            throw new ApplicationException("Could not find a certificate with thumbprint '" + thumbprint + "'");
        }

        private static X509Certificate2 TryGetClientCertificateFromStore(StoreLocation location, string thumbprint)
        {
            X509Store store = null;
            try
            {
                store = new X509Store(StoreName.My, location);
                store.Open(OpenFlags.OpenExistingOnly | OpenFlags.ReadOnly);
                var certs = store.Certificates.Find(X509FindType.FindByThumbprint, thumbprint, false);

                if (certs.Count == 1)
                {
                    var cert = certs[0];
                    return cert;
                }
                return null;
            }
            finally
            {
                if (store != null)
                    store.Close();
            }
        }
    }
}
